{
TWebControl and TWebChromium classes

  ES: Clase para la manipulacin del contenido de un navegador TWebBrowser o TChromium
  EN: Class for management the contents of a browser TWebBrowser or TChromium

=========================================================================
History:

ver 0.1.9
  ES:
    nuevo: documentacin
    nuevo: se hace compatible con el navegador chromium y FireMonkey
  EN:
    new: documentation
    new: now compatible with chromium browser and FireMonkey
=========================================================================
IMPORTANTE PROGRAMADORES: Por favor, si tienes comentarios, mejoras,
  ampliaciones, errores y/o cualquier otro tipo de sugerencia, envame un correo a:
  gmlib@cadetill.com

IMPORTANT DEVELOPERS: please, if you have comments, improvements, enlargements,
  errors and/or any another type of suggestion, please send me a mail to:
  gmlib@cadetill.com
}
{*------------------------------------------------------------------------------
  The WebControlVCL unit includes the necessary classes to encapsulate the access to a browser with VCL framework.
  This browser can be a TWebBrowser or TChromium.
  You can de/activate this browsers into the gmlib.inc file.
  By default, only the TWebBrowser is active.

  @author Xavier Martinez (cadetill)
  @version 0.1.9
-------------------------------------------------------------------------------}
{=------------------------------------------------------------------------------
  La unit WebControlVCL incluye las clases necesarias para encapsular el acceso a un navegador mediante el framework de la VCL.
  Este navegador puede ser un TWebBrowser o TChromium.
  Puedes des/activar estos navegadores desde el archivo gmlib.inc.
  Por defecto, slo est activo el TWebBrowser.

  @author Xavier Martinez (cadetill)
  @version 0.1.9
-------------------------------------------------------------------------------}
unit WebControlVCL;

{.$DEFINE WEBBROWSER}
{.$DEFINE CHROMIUM}
{$I ..\gmlib.inc}

interface

uses
  {$IFDEF WEBBROWSER}
  MSHTML, SHDocVw,
  {$ENDIF}

  {$IFDEF CHROMIUM}
  ceflib, cefvcl,
  {$ENDIF}

  {$IF CompilerVersion < 23}  // ES: si la versin es inferior a la XE2 - EN: if lower than XE2 version
  SysUtils,
  {$ELSE}                     // ES: si la verisn es la XE2 o superior - EN: if version is XE2 or higher
  System.SysUtils,
  {$IFEND}

  WebControl;

type
  {*------------------------------------------------------------------------------
    TWebControl class is a specialization of TCustomWeb for a TWebBrowser browser.
    For that the compiler take into account this class, must be activated in the gmlib.inc file.
  -------------------------------------------------------------------------------}
  {=------------------------------------------------------------------------------
    La clase TWebControl es una especializacin de TCustomWeb para el navegador TWebBrowser.
    Para que el compilador tenga en cuenta esta clase debes activarla en el fichero gmlib.inc
  -------------------------------------------------------------------------------}
  {$IFDEF WEBBROWSER}
  TWebControl = class(TCustomWeb)
  protected
    {*------------------------------------------------------------------------------
      WebFormGet function return the form specified by FormNumber
      @param FormNumber index to form to return
      @return the form in IHTMLFormElement format
    -------------------------------------------------------------------------------}
    {=------------------------------------------------------------------------------
      La funcin WebFormGet devuelve el formulario especificado por FormNumber
      @param FormNumber ndice del formulario a devolver
      @return el formulario en formato IHTMLFormElement
    -------------------------------------------------------------------------------}
    function WebFormGet(const FormNumber: Integer): IHTMLFormElement;
    function WebFormFieldValue(const FormIdx: Integer; const FieldName: string): string; overload; override;
  public
    {*------------------------------------------------------------------------------
      Constructor of the class
      @param WebBrowser is the browser to manipulate
    -------------------------------------------------------------------------------}
    {=------------------------------------------------------------------------------
      Constructor de la clase
      @param WebBrowser es el navegador a manipular
    -------------------------------------------------------------------------------}
    constructor Create(WebBrowser: TWebBrowser); reintroduce; virtual;

    procedure WebFormNames; override;
    procedure WebFormFields(const FormIdx: Integer); overload; override;
    procedure WebFormSetFieldValue(const FormIdx: Integer; const FieldName, NewValue: string); overload; override;
    procedure WebFormSubmit(const FormIdx: Integer); overload; override;
    function WebHTMLCode: string; override;
    procedure WebPrintWithoutDialog; override;
    procedure WebPrintWithDialog; override;
    procedure WebPrintPageSetup; override;
    procedure WebPreview; override;
    procedure SaveToJPGFile(FileName: TFileName = ''); override;
    {*------------------------------------------------------------------------------
      Set a browser
      @param Browser browser to set
    -------------------------------------------------------------------------------}
    {=------------------------------------------------------------------------------
      Establece un navegador
      @param Browser navegador a establecer
    -------------------------------------------------------------------------------}
    procedure SetBrowser(Browser: TWebBrowser); reintroduce; virtual;
  end;
  {$ENDIF}

  {*------------------------------------------------------------------------------
    TWebChromium class is a specialization of TCustomWeb for a TChromium browser.
    For that the compiler take into account this class, must be activated in the file gmlib.inc
  -------------------------------------------------------------------------------}
  {=------------------------------------------------------------------------------
    La clase TWebChromium es una especializacin de TCustomWeb para el navegador TChromium.
    Para que el compilador tenga en cuenta esta clase debes activarla en el fichero gmlib.inc
  -------------------------------------------------------------------------------}
  {$IFDEF CHROMIUM}
  TWebChromium = class(TCustomWebChromium)
  protected
    function WebFormFieldValue(const FormIdx: Integer; const FieldName: string): string; overload; override;
  public
    {*------------------------------------------------------------------------------
      Constructor of the class
      @param WebBrowser is the browser to manipulate
    -------------------------------------------------------------------------------}
    {=------------------------------------------------------------------------------
      Constructor de la clase
      @param WebBrowser es el navegador a manipular
    -------------------------------------------------------------------------------}
    constructor Create(WebBrowser: TChromium); reintroduce; virtual;

    procedure WebFormNames; override;
    procedure WebFormFields(const FormIdx: Integer); overload; override;
    procedure WebFormSetFieldValue(const FormIdx: Integer;
      const FieldName, NewValue: string); overload; override;
    procedure WebFormSubmit(const FormIdx: Integer); overload; override;
    function WebHTMLCode: string; override;
    procedure WebPrintWithoutDialog; override;
    procedure WebPrintWithDialog; override;
    procedure WebPrintPageSetup; override;
    procedure WebPreview; override;
    procedure SaveToJPGFile(FileName: TFileName = ''); override;
    {*------------------------------------------------------------------------------
      Set a browser
      @param Browser browser to set
    -------------------------------------------------------------------------------}
    {=------------------------------------------------------------------------------
      Establece un navegador
      @param Browser navegador a establecer
    -------------------------------------------------------------------------------}
    procedure SetBrowser(Browser: TChromium); reintroduce; virtual;
  end;
  {$ENDIF}

implementation

uses
  {$IFDEF WEBBROWSER}
    {$IF CompilerVersion < 23}
    ActiveX, Types, Graphics, Forms,
    {$ELSE}
    Winapi.ActiveX, System.Types, Vcl.Graphics, Vcl.Forms,
    {$IFEND}
  {$ENDIF}

  {$IF CompilerVersion < 23}
  jpeg, DateUtils;
  {$ELSE}
  Vcl.Imaging.jpeg, System.DateUtils;
  {$IFEND}

{ TWebControl }

{$IFDEF WEBBROWSER}
constructor TWebControl.Create(WebBrowser: TWebBrowser);
begin
  inherited Create(WebBrowser);
end;

procedure TWebControl.SaveToJPGFile(FileName: TFileName);
var
  viewObject: IViewObject;
  r: TRect;
  {$IF CompilerVersion < 23}
  bitmap: Graphics.TBitmap;
  {$ELSE}
  bitmap: Vcl.Graphics.TBitmap;
  {$IFEND}
begin
  if not Assigned(FWebBrowser) then
    raise Exception.Create('WebBrowser not assigned');
  if not (FWebBrowser is TWebBrowser) then
    raise Exception.Create('The WebBrowser property is not a TWebBrowser.');

  TWebBrowser(FWebBrowser).Document.QueryInterface(IViewObject, viewObject);
  if Assigned(viewObject) then
  try
    {$IF CompilerVersion < 23}
    bitmap := Graphics.TBitmap.Create;
    {$ELSE}
    bitmap := Vcl.Graphics.TBitmap.Create;
    {$IFEND}
    try
      r := Rect(0, 0, TWebBrowser(FWebBrowser).Width, TWebBrowser(FWebBrowser).Height);

      bitmap.Height := TWebBrowser(FWebBrowser).Height;
      bitmap.Width := TWebBrowser(FWebBrowser).Width;

      {$IF CompilerVersion < 23}
      viewObject.Draw(DVASPECT_CONTENT, 1, nil, nil, Application.Handle, bitmap.Canvas.Handle, @r, nil, nil, 0);
      {$ELSE}
      viewObject.Draw(DVASPECT_CONTENT, 1, nil, nil, Vcl.Forms.Application.Handle, bitmap.Canvas.Handle, @r, nil, nil, 0);
      {$IFEND}

      with TJPEGImage.Create do
      try
        Assign(bitmap);
        SaveToFile(FileName);
      finally
        Free;
      end;
    finally
      bitmap.Free;
    end;
  finally
    viewObject._Release;
  end;
end;

procedure TWebControl.SetBrowser(Browser: TWebBrowser);
begin
  inherited SetBrowser(Browser);
end;

procedure TWebControl.WebFormFields(const FormIdx: Integer);
var
  Form: IHTMLFormElement;
  Fld: IHTMLElement;
  FName: string;
  Idx: integer;
begin
  Fields.Clear;
  if not Assigned(FWebBrowser) then
    raise Exception.Create('WebBrowser not assigned');
  if not (FWebBrowser is TWebBrowser) then
    raise Exception.Create('The WebBrowser property is not a TWebBrowser.');
  if FForms.Count = 0 then
    raise Exception.Create('Property Forms not initialized or empty.');
  if (FormIdx < 0) or (FormIdx >= FForms.Count) then
    raise Exception.Create('Index out of bounds.');

  Form := WebFormGet(FormIdx);

  Fields.Clear;
  for idx := 0 to form.length - 1 do
  begin
    Fld := Form.Item(Idx, '') as IHTMLElement;

    if Fld = nil then Continue;
    FName := Fld.id;
    if Fld.tagName = 'INPUT' then FName := (Fld as IHTMLInputElement).name;
    if Fld.tagName = 'SELECT' then FName := (Fld as IHTMLSelectElement).name;
    if Fld.tagName = 'TEXTAREA' then FName := (Fld as IHTMLTextAreaElement).name;

    Fields.Add(FName) ;
  end;
end;

function TWebControl.WebFormFieldValue(const FormIdx: Integer;
  const FieldName: string): string;
var
  Form: IHTMLFormElement;
  Field: IHTMLElement;
begin
  Result := '';

  if not Assigned(FWebBrowser) then
    raise Exception.Create('WebBrowser not assigned');
  if not (FWebBrowser is TWebBrowser) then
    raise Exception.Create('The WebBrowser property is not a TWebBrowser.');

  Form := WebFormGet(FormIdx);
  if not Assigned(Form) then Exit;
  Field := Form.Item(FieldName,'') as IHTMLElement;
  if not Assigned(Field) then Exit;

  if Field.tagName = 'INPUT' then Result := (Field as IHTMLInputElement).value;
  if Field.tagName = 'SELECT' then Result := (Field as IHTMLSelectElement).value;
  if Field.tagName = 'TEXTAREA' then Result := (Field as IHTMLTextAreaElement).value;
end;

function TWebControl.WebFormGet(const FormNumber: Integer): IHTMLFormElement;
var
  Forms: IHTMLElementCollection;
begin
  Result := nil;
  if not Assigned(FWebBrowser) then
    raise Exception.Create('WebBrowser not assigned');
  if FWebBrowser is TWebBrowser then
  begin
    Forms := (TWebBrowser(FWebBrowser).Document as IHTMLDocument2).forms as IHTMLElementCollection;
    Result := Forms.Item(FormNumber, '') as IHTMLFormElement;
  end;
end;

procedure TWebControl.WebFormNames;
var
  Frms: IHTMLElementCollection;
  Frm: IHTMLFormElement;
  Idx: integer;
begin
  FForms.Clear;
  if not Assigned(FWebBrowser) then
    raise Exception.Create('WebBrowser not assigned');
  if not (FWebBrowser is TWebBrowser) then
    raise Exception.Create('The WebBrowser property is not a TWebBrowser.');

  Frms := (TWebBrowser(FWebBrowser).Document as IHTMLDocument2).forms as IHTMLElementCollection;
  for Idx := 0 to Frms.Length - 1 do
  begin
    Frm := Frms.Item(Idx,0) as IHTMLFormElement;
    FForms.Add(Frm.Name);
  end;
end;

procedure TWebControl.WebFormSetFieldValue(const FormIdx: Integer;
  const FieldName, NewValue: string);
var
  Form: IHTMLFormElement;
  Field: IHTMLElement;
begin
  inherited;

  if not Assigned(FWebBrowser) then
    raise Exception.Create('WebBrowser not assigned');
  if not (FWebBrowser is TWebBrowser) then
    raise Exception.Create('The WebBrowser property is not a TWebBrowser.');
  if (FormIdx < 0) or (FormIdx >= FForms.Count) then
    raise Exception.Create('Index out of bounds.');

  Form := WebFormGet(FormIdx);
  Field := Form.Item(FieldName, '') as IHTMLElement;
  if Field = nil then Exit;

  if Field.tagName = 'INPUT' then (Field as IHTMLInputElement).value := NewValue;
  if Field.tagName = 'SELECT' then (Field as IHTMLSelectElement).value := NewValue;
  if Field.tagName = 'TEXTAREA' then (Field as IHTMLTextAreaElement).value := NewValue;
end;

procedure TWebControl.WebFormSubmit(const FormIdx: Integer);
var
  Form: IHTMLFormElement;
begin
  inherited;

  if not Assigned(FWebBrowser) then
    raise Exception.Create('WebBrowser not assigned');
  if not (FWebBrowser is TWebBrowser) then
    raise Exception.Create('The WebBrowser property is not a TWebBrowser.');

  Form := WebFormGet(FormIdx);

  Form.submit;
end;

function TWebControl.WebHTMLCode: string;
begin
  if not Assigned(FWebBrowser) then
    raise Exception.Create('WebBrowser not assigned');
  if not (FWebBrowser is TWebBrowser) then
    raise Exception.Create('The WebBrowser property is not a TWebBrowser.');

  Result := IHTMLDocument2(TWebBrowser(FWebBrowser).Document).body.innerHTML;
end;

procedure TWebControl.WebPreview;
var
  vIn, vOut: OleVariant;
begin
  if not Assigned(FWebBrowser) then
    raise Exception.Create('WebBrowser not assigned');
  if not (FWebBrowser is TWebBrowser) then
    raise Exception.Create('The WebBrowser property is not a TWebBrowser.');

  TWebBrowser(FWebBrowser).ControlInterface.ExecWB(OLECMDID_PRINTPREVIEW, OLECMDEXECOPT_DONTPROMPTUSER, vIn, vOut) ;
end;

procedure TWebControl.WebPrintPageSetup;
var
  vIn, vOut: OleVariant;
begin
  if not Assigned(FWebBrowser) then
    raise Exception.Create('WebBrowser not assigned');
  if not (FWebBrowser is TWebBrowser) then
    raise Exception.Create('The WebBrowser property is not a TWebBrowser.');

  TWebBrowser(FWebBrowser).ControlInterface.ExecWB(OLECMDID_PAGESETUP, OLECMDEXECOPT_PROMPTUSER, vIn, vOut) ;
end;

procedure TWebControl.WebPrintWithDialog;
var
  vIn, vOut: OleVariant;
begin
  if not Assigned(FWebBrowser) then
    raise Exception.Create('WebBrowser not assigned');
  if not (FWebBrowser is TWebBrowser) then
    raise Exception.Create('The WebBrowser property is not a TWebBrowser.');

  TWebBrowser(FWebBrowser).ControlInterface.ExecWB(OLECMDID_PRINT, OLECMDEXECOPT_PROMPTUSER, vIn, vOut) ;
end;

procedure TWebControl.WebPrintWithoutDialog;
var
  vIn, vOut: OleVariant;
begin
  if not Assigned(FWebBrowser) then
    raise Exception.Create('WebBrowser not assigned');
  if not (FWebBrowser is TWebBrowser) then
    raise Exception.Create('The WebBrowser property is not a TWebBrowser.');

  TWebBrowser(FWebBrowser).ControlInterface.ExecWB(OLECMDID_PRINT, OLECMDEXECOPT_DONTPROMPTUSER, vIn, vOut) ;
end;
{$ENDIF}

{ TWebChromium }

{$IFDEF CHROMIUM}
constructor TWebChromium.Create(WebBrowser: TChromium);
begin
  inherited Create(WebBrowser);
end;

procedure TWebChromium.SaveToJPGFile(FileName: TFileName);
var
   bmp: Vcl.Graphics.TBitmap;
begin
  if not Assigned(FWebBrowser) then
    raise Exception.Create('WebBrowser not assigned');
  if not (FWebBrowser is TChromium) then
    raise Exception.Create('The WebBrowser property is not a TChromium.');

  bmp := Vcl.Graphics.TBitMap.Create;
  try
    cefvcl.CefGetBitmap(TChromium(FWebBrowser).Browser, PET_VIEW, bmp);
    with TJPEGImage.Create do
    try
      Assign(bmp);
      SaveToFile(FileName);
    finally
      Free;
    end;
  finally
    FreeAndNil(bmp);
  end;
end;

procedure TWebChromium.SetBrowser(Browser: TChromium);
begin
  FWebBrowser := Browser;
end;

procedure TWebChromium.WebFormFields(const FormIdx: Integer);
var
  Finish: Boolean;
  FormName: string;
begin
  Fields.Clear;
  if not Assigned(FWebBrowser) then
    raise Exception.Create('WebBrowser not assigned');
  if not (FWebBrowser is TChromium) then
    raise Exception.Create('The WebBrowser property is not a TChromium.');
  if FForms.Count = 0 then
    raise Exception.Create('Property Forms not initialized or empty.');
  if (FormIdx < 0) or (FormIdx >= FForms.Count) then
    raise Exception.Create('Index out of bounds.');

  FormName := FForms[FormIdx];
  Finish := False;

  TChromium(FWebBrowser).Browser.MainFrame.VisitDomProc(
    procedure (const doc: ICefDomDocument)
    begin
      FFields.CommaText := GetFieldsName(doc.GetElementById(FormName));
      Finish := True;
    end
  );

  repeat Application.ProcessMessages until (Finish);
end;

function TWebChromium.WebFormFieldValue(const FormIdx: Integer;
  const FieldName: string): string;
var
  Finish: Boolean;
  FormName: string;
  Temp: string;
begin
  inherited;

  Temp := '';

  if not Assigned(FWebBrowser) then
    raise Exception.Create('WebBrowser not assigned');
  if not (FWebBrowser is TChromium) then
    raise Exception.Create('The WebBrowser property is not a TChromium.');
  if FForms.Count = 0 then
    raise Exception.Create('Property Forms not initialized or empty.');
  if (FormIdx < 0) or (FormIdx >= FForms.Count) then
    raise Exception.Create('Index out of bounds.');

  FormName := FForms[FormIdx];

  TChromium(FWebBrowser).Browser.MainFrame.VisitDomProc(
    procedure (const doc: ICefDomDocument)
    begin
      Temp := GetFieldValue(doc.GetElementById(FormName), FieldName);
      Finish := True;
    end
  );

  repeat Application.ProcessMessages until (Finish);
  Result := Temp;
end;

procedure TWebChromium.WebFormNames;
var
  Finish: Boolean;
  EndTime: TTime;
begin
  FForms.Clear;
  if not Assigned(FWebBrowser) then
    raise Exception.Create('WebBrowser not assigned');
  if not (FWebBrowser is TChromium) then
    raise Exception.Create('The WebBrowser property is not a TChromium.');

  Finish := False;
  TChromium(FWebBrowser).Browser.MainFrame.VisitDomProc(
        procedure (const doc: ICefDomDocument)
        begin
          FForms.CommaText := GetFormsName(doc.Body);
          Finish := True;
        end
  );
  EndTime := IncSecond(Time, 4);

  repeat Sleep(1); Application.ProcessMessages; until Finish or (Time > EndTime);
  if Time > EndTime then
    raise Exception.Create('Time out');
end;

procedure TWebChromium.WebFormSetFieldValue(const FormIdx: Integer;
  const FieldName, NewValue: string);
var
  Finish: Boolean;
  FormName: string;
begin
  inherited;

  if not Assigned(FWebBrowser) then
    raise Exception.Create('WebBrowser not assigned');
  if not (FWebBrowser is TChromium) then
    raise Exception.Create('The WebBrowser property is not a TChromium.');
  if FForms.Count = 0 then
    raise Exception.Create('Property Forms not initialized or empty.');
  if (FormIdx < 0) or (FormIdx >= FForms.Count) then
    raise Exception.Create('Index out of bounds.');

  FormName := FForms[FormIdx];

  TChromium(FWebBrowser).Browser.MainFrame.VisitDomProc(
    procedure (const doc: ICefDomDocument)
    begin
      SetFieldValue(doc.GetElementById(FormName), FieldName, NewValue);
      Finish := True;
    end
  );

  repeat Application.ProcessMessages until (Finish);
end;

procedure TWebChromium.WebFormSubmit(const FormIdx: Integer);
var
  FormName: string;
begin
  inherited;

  if not Assigned(FWebBrowser) then
    raise Exception.Create('WebBrowser not assigned');
  if not (FWebBrowser is TChromium) then
    raise Exception.Create('The WebBrowser property is not a TChromium.');
  if FForms.Count = 0 then
    raise Exception.Create('Property Forms not initialized or empty.');
  if (FormIdx < 0) or (FormIdx >= FForms.Count) then
    raise Exception.Create('Index out of bounds.');

  FormName := FForms[FormIdx];

  if TChromium(FWebBrowser).Browser <> nil then
    TChromium(FWebBrowser).Browser.MainFrame.ExecuteJavaScript(
      'document.forms["' + FormName + '"].submit();', '', 0);
end;

function TWebChromium.WebHTMLCode: string;
begin
  Result := '';
  if not Assigned(FWebBrowser) then
    raise Exception.Create('WebBrowser not assigned');
  if not (FWebBrowser is TChromium) then
    raise Exception.Create('The WebBrowser property is not a TChromium.');

  if TChromium(FWebBrowser).Browser = nil then Exit;
  Result := TChromium(FWebBrowser).Browser.MainFrame.Source;
end;

procedure TWebChromium.WebPreview;
begin
  raise Exception.Create('This method is not implemented for this class. Call WebPrintWithDialog in his stead.');
end;

procedure TWebChromium.WebPrintPageSetup;
begin
  raise Exception.Create('This method is not implemented for this class. Call WebPrintWithDialog in his stead.');
end;

procedure TWebChromium.WebPrintWithDialog;
begin
  if not Assigned(FWebBrowser) then
    raise Exception.Create('WebBrowser not assigned');
  if not (FWebBrowser is TChromium) then
    raise Exception.Create('The WebBrowser property is not a TChromium.');

  if TChromium(FWebBrowser).Browser <> nil then
    TChromium(FWebBrowser).Browser.MainFrame.Print;
end;

procedure TWebChromium.WebPrintWithoutDialog;
begin
  raise Exception.Create('This method is not implemented for this class. Call WebPrintWithDialog in his stead.');
end;
{$ENDIF}

end.
